@@ -27,7 +27,7 @@ import codecs
 import platform
 import importlib
 from gzip import GzipFile
-from base64 import b64decode, b64encode
+from base64 import b64encode
 from decimal import Context, Decimal, Inexact
 from hashlib import sha1
 from collections import namedtuple
@@ -501,7 +501,26 @@ else:
 
 
 if PY2:
+    # Under Python 2, math.floor and math.ceil return floats, which cannot
+    # represent large integers - eg `float(2**53) == float(2**53 + 1)`.
+    # We therefore implement them entirely in (long) integer operations.
     def floor(x):
-        return int(math.floor(x))
+        if int(x) != x and x < 0:
+            return int(x) - 1
+        return int(x)
+
+    def ceil(x):
+        if int(x) != x and x > 0:
+            return int(x) + 1
+        return int(x)
 else:
     floor = math.floor
+    ceil = math.ceil
+
+
+if PY2:
+    def b64decode(s):
+        from base64 import b64decode as base
+        return hbytes(base(s))
+else:
+    from base64 import b64decode
